@@ -5,12 +5,10 @@
  * Copyright (C) 2007, 2008 Red Hat, Inc.
  */
 
-#include <stdio.h>
 #include <string.h>
 
 #include "soup-message-headers.h"
-#include "soup-headers.h"
-#include "soup-misc.h"
+#include "soup.h"
 
 /**
  * SECTION:soup-message-headers
@@ -21,6 +19,12 @@
  * with a request or response.
  **/
 
+/**
+ * SoupMessageHeaders:
+ *
+ * The HTTP message headers associated with a request or response.
+ */
+
 /**
  * SoupMessageHeadersType:
  * @SOUP_MESSAGE_HEADERS_REQUEST: request headers
@@ -97,26 +101,12 @@ soup_message_headers_free (SoupMessageHeaders *hdrs)
 	if (--hdrs->ref_count == 0) {
 		soup_message_headers_clear (hdrs);
 		g_array_free (hdrs->array, TRUE);
-		if (hdrs->concat)
-			g_hash_table_destroy (hdrs->concat);
+		g_clear_pointer (&hdrs->concat, g_hash_table_destroy);
 		g_slice_free (SoupMessageHeaders, hdrs);
 	}
 }
 
-GType
-soup_message_headers_get_type (void)
-{
-	static volatile gsize type_volatile = 0;
-
-	if (g_once_init_enter (&type_volatile)) {
-		GType type = g_boxed_type_register_static (
-			g_intern_static_string ("SoupMessageHeaders"),
-			(GBoxedCopyFunc) soup_message_headers_copy,
-			(GBoxedFreeFunc) soup_message_headers_free);
-		g_once_init_leave (&type_volatile, type);
-	}
-	return type_volatile;
-}
+G_DEFINE_BOXED_TYPE (SoupMessageHeaders, soup_message_headers, soup_message_headers_copy, soup_message_headers_free)
 
 /**
  * soup_message_headers_clear:
@@ -522,7 +512,7 @@ soup_message_headers_foreach (SoupMessageHeaders *hdrs,
 }
 
 
-static GStaticMutex header_pool_mutex = G_STATIC_MUTEX_INIT;
+G_LOCK_DEFINE_STATIC (header_pool);
 static GHashTable *header_pool, *header_setters;
 
 static void transfer_encoding_setter (SoupMessageHeaders *, const char *);
@@ -549,7 +539,7 @@ intern_header_name (const char *name, SoupHeaderSetter *setter)
 {
 	const char *interned;
 
-	g_static_mutex_lock (&header_pool_mutex);
+	G_LOCK (header_pool);
 
 	if (!header_pool) {
 		header_pool = g_hash_table_new (soup_str_case_hash, soup_str_case_equal);
@@ -572,7 +562,7 @@ intern_header_name (const char *name, SoupHeaderSetter *setter)
 	if (setter)
 		*setter = g_hash_table_lookup (header_setters, interned);
 
-	g_static_mutex_unlock (&header_pool_mutex);
+	G_UNLOCK (header_pool);
 	return interned;
 }
 
@@ -734,8 +724,13 @@ soup_message_headers_set_encoding (SoupMessageHeaders *hdrs,
 goffset
 soup_message_headers_get_content_length (SoupMessageHeaders *hdrs)
 {
-	return (hdrs->encoding == SOUP_ENCODING_CONTENT_LENGTH) ?
-		hdrs->content_length : 0;
+	SoupEncoding encoding;
+
+	encoding = soup_message_headers_get_encoding (hdrs);
+	if (encoding == SOUP_ENCODING_CONTENT_LENGTH)
+		return hdrs->content_length;
+	else
+		return 0;
 }
 
 /**
@@ -761,8 +756,8 @@ soup_message_headers_set_content_length (SoupMessageHeaders *hdrs,
 {
 	char length[128];
 
-	snprintf (length, sizeof (length), "%" G_GUINT64_FORMAT,
-		  content_length);
+	g_snprintf (length, sizeof (length), "%" G_GUINT64_FORMAT,
+		    content_length);
 	soup_message_headers_remove (hdrs, "Transfer-Encoding");
 	soup_message_headers_replace (hdrs, "Content-Length", length);
 }
@@ -839,17 +834,17 @@ soup_message_headers_set_expectations (SoupMessageHeaders *hdrs,
  * Represents a byte range as used in the Range header.
  *
  * If @end is non-negative, then @start and @end represent the bounds
- * of of the range, counting from %0. (Eg, the first 500 bytes would be
- * represented as @start = %0 and @end = %499.)
+ * of of the range, counting from 0. (Eg, the first 500 bytes would be
+ * represented as @start = 0 and @end = 499.)
  *
- * If @end is %-1 and @start is non-negative, then this represents a
+ * If @end is -1 and @start is non-negative, then this represents a
  * range starting at @start and ending with the last byte of the
  * requested resource body. (Eg, all but the first 500 bytes would be
- * @start = %500, and @end = %-1.)
+ * @start = 500, and @end = -1.)
  *
- * If @end is %-1 and @start is negative, then it represents a "suffix
+ * If @end is -1 and @start is negative, then it represents a "suffix
  * range", referring to the last -@start bytes of the resource body.
- * (Eg, the last 500 bytes would be @start = %-500 and @end = %-1.)
+ * (Eg, the last 500 bytes would be @start = -500 and @end = -1.)
  *
  * Since: 2.26
  **/
@@ -1212,8 +1207,9 @@ content_type_setter (SoupMessageHeaders *hdrs, const char *value)
 /**
  * soup_message_headers_get_content_type:
  * @hdrs: a #SoupMessageHeaders
- * @params: (out) (allow-none) (transfer full): return location for
- * the Content-Type parameters (eg, "charset"), or %NULL
+ * @params: (out) (element-type utf8 utf8) (allow-none) (transfer full):
+ *   return location for the Content-Type parameters (eg, "charset"), or
+ *   %NULL
  *
  * Looks up the "Content-Type" header in @hdrs, parses it, and returns
  * its value in *@content_type and *@params. @params can be %NULL if you
