@@ -5,14 +5,12 @@
  * Copyright 1999-2003 Ximian, Inc.
  */
 
-#include <ctype.h>
 #include <string.h>
 #include <stdlib.h>
 
 #include "soup-uri.h"
+#include "soup.h"
 #include "soup-misc-private.h"
-#include "soup-form.h"
-#include "soup-misc.h"
 
 /**
  * SECTION:soup-uri
@@ -696,11 +694,11 @@ uri_normalized_copy (const char *part, int length,
 	gboolean need_fixup = FALSE;
 
 	s = d = (unsigned char *)normalized;
-	do {
+	while (*s) {
 		if (*s == '%') {
 			if (!g_ascii_isxdigit (s[1]) ||
 			    !g_ascii_isxdigit (s[2])) {
-				*d++ = *s;
+				*d++ = *s++;
 				continue;
 			}
 
@@ -708,7 +706,7 @@ uri_normalized_copy (const char *part, int length,
 			if (soup_char_is_uri_unreserved (c) ||
 			    (unescape_extra && strchr (unescape_extra, c))) {
 				*d++ = c;
-				s += 2;
+				s += 3;
 			} else {
 				/* We leave it unchanged. We used to uppercase percent-encoded
 				 * triplets but we do not do it any more as RFC3986 Section 6.2.2.1
@@ -716,14 +714,15 @@ uri_normalized_copy (const char *part, int length,
 				 */
 				*d++ = *s++;
 				*d++ = *s++;
-				*d++ = *s;
+				*d++ = *s++;
 			}
 		} else {
 			if (!g_ascii_isgraph (*s))
 				need_fixup = TRUE;
-			*d++ = *s;
+			*d++ = *s++;
 		}
-	} while (*s++);
+	}
+	*d = '\0';
 
 	if (need_fixup) {
 		GString *fixed;
@@ -1220,18 +1219,4 @@ soup_uri_host_equal (gconstpointer v1, gconstpointer v2)
 	return g_ascii_strcasecmp (one->host, two->host) == 0;
 }
 
-
-GType
-soup_uri_get_type (void)
-{
-	static volatile gsize type_volatile = 0;
-
-	if (g_once_init_enter (&type_volatile)) {
-		GType type = g_boxed_type_register_static (
-			g_intern_static_string ("SoupURI"),
-			(GBoxedCopyFunc) soup_uri_copy,
-			(GBoxedFreeFunc) soup_uri_free);
-		g_once_init_leave (&type_volatile, type);
-	}
-	return type_volatile;
-}
+G_DEFINE_BOXED_TYPE (SoupURI, soup_uri, soup_uri_copy, soup_uri_free)
